@@ -20,7 +20,7 @@
  *    Lauris Kaplinski <lauris@ximian.com>
  *    Jose M. Celorio <chema@ximian.com>
  *
- *  Copyright (C) 2000-2001 Ximian, Inc. and Jose M. Celorio
+ *  Copyright (C) 2000-2003 Ximian, Inc.
  *
  */
 
@@ -30,12 +30,11 @@
 #include <string.h>
 #include <ctype.h>
 #include <locale.h>
+#include "gpa-root.h"
 #include "gpa-utils.h"
 #include "gpa-config.h"
 #include "gpa-node-private.h"
 
-/* GPANode */
-
 enum {MODIFIED, LAST_SIGNAL};
 static GObjectClass *parent_class;
 static guint node_signals [LAST_SIGNAL] = {0};
@@ -86,8 +85,9 @@ gpa_node_class_init (GPANodeClass *klass)
 static void
 gpa_node_init (GPANode *node)
 {
-	node->parent = NULL;
-	node->next   = NULL;
+	node->parent   = NULL;
+	node->next     = NULL;
+	node->children = NULL;
 }
 
 static void
@@ -120,7 +120,7 @@ gpa_node_new (GType type, const guchar *id)
 	node = g_object_new (type, NULL);
 
 	if (id)
-		node->qid = gpa_quark_from_string (id);
+		node->qid = g_quark_from_string (id);
 
 	return node;
 }
@@ -169,7 +169,7 @@ gpa_node_get_value (GPANode *node)
 		return GPA_NODE_GET_CLASS (node)->get_value (node);
 	else
 		g_warning ("Can't get_valued from \"%s\" node because the \"%s\" Class does not have a "
-			   "get_value method.", gpa_node_id (node), G_OBJECT_TYPE_NAME (node));
+			   "get_value method.", GPA_NODE_ID (node), G_OBJECT_TYPE_NAME (node));
 
 	return NULL;
 }
@@ -177,38 +177,85 @@ gpa_node_get_value (GPANode *node)
 GPANode *
 gpa_node_get_child (GPANode *node, GPANode *previous_child)
 {
+	GPANode *child;
+
+	/* FIXME: Handle GPAReference */
 	g_return_val_if_fail (node != NULL, NULL);
 	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
 	g_return_val_if_fail (!previous_child || GPA_IS_NODE (previous_child), NULL);
 
-	if (GPA_NODE_GET_CLASS (node)->get_child)
-		return GPA_NODE_GET_CLASS (node)->get_child (node, previous_child);
-	/* Not all nodes have childs (GPAValue for example), method is optional */
+	if (previous_child)
+		child = previous_child->next;
+	else
+		child = node->children;
 
-	return NULL;
+	if (child)
+		gpa_node_ref (child);
+
+	return child;
+}	
+
+static GPANode *
+gpa_node_lookup_real (GPANode *node, guchar *path)
+{
+	GPANode *child;
+	const guchar *next;
+	guchar *dot;
+
+	g_assert (node);
+	g_assert (path);
+	
+	dot = strchr (path, '.');
+	if (dot != NULL) {
+		next = dot + 1;
+		*dot = 0; /* allows us to compare with quarks */
+	} else {
+		next = NULL;
+	}
+
+	child = GPA_NODE (node)->children;
+	while (child) {
+		if (GPA_NODE_ID_COMPARE (child, path))
+		    break;
+		child = child->next;
+	}
+	if (next)
+		*dot = '.';
+
+	if (!child) {
+		return NULL;
+	}
+
+	if (!next) {
+		gpa_node_ref (child);
+		return child;
+	}
+
+	return gpa_node_lookup (child, next);
 }
 
 GPANode *
 gpa_node_lookup (GPANode *node, const guchar *path)
 {
-	GPANode *ret = NULL;
+	GPANode *ret;
+	gchar *p;
 	
+	g_return_val_if_fail (path != NULL, NULL);
+	g_return_val_if_fail (*path != '\0', NULL);
+
+	if (node == NULL) {
+		node = gpa_root;
+	}
+	if (GPA_IS_REFERENCE (node)) {
+		node = GPA_REFERENCE_REFERENCE (node);
+	}
+
 	g_return_val_if_fail (node != NULL, NULL);
 	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
-	g_return_val_if_fail (path != NULL, NULL);
-	g_return_val_if_fail (!*path || isalnum (*path), NULL);
 
-	/* FIMXE: Why do we allow lookup without a path? (Chema) */
-	if (!*path) {
-		gpa_node_ref (node);
-		return node;
-	}
-	
-	if (GPA_NODE_GET_CLASS (node)->lookup)
-		ret = GPA_NODE_GET_CLASS (node)->lookup (node, path);
-	else
-		g_warning ("Can't do a node_lookup of \"%s\" with the \"%s\" path node because the \"%s\" Class does not "
-			   "have a _lookup method.", gpa_node_id (node), path, G_OBJECT_TYPE_NAME (node));
+	p = g_strdup (path);
+	ret = gpa_node_lookup_real (node, p);
+	g_free (p);
 
 	return ret;
 }
@@ -224,9 +271,8 @@ gpa_node_set_value (GPANode *node, const guchar *value)
 
 	if (GPA_NODE_GET_CLASS (node)->set_value) {
 		ret = GPA_NODE_GET_CLASS (node)->set_value (node, value);
-		gpa_enter ("Request modified from gpa_node_set_value Node:[%d]", GPOINTER_TO_INT (node));
-		gpa_node_request_modified (node, GPA_NODE_MODIFIED_FLAG);
-		gpa_leave ("Request modified from gpa_node_set_value");
+		if (ret)
+			gpa_node_emit_modified (node);
 	} else {
 		g_warning ("Can't set_valued of \"%s\" to \"%s\" because the \"%s\" Class does not have a "
 			   "set_value method.", gpa_node_id (node), value, G_OBJECT_TYPE_NAME (node));
@@ -235,85 +281,13 @@ gpa_node_set_value (GPANode *node, const guchar *value)
 	return ret;
 }
 
-/* Signal stuff */
 void
-gpa_node_emit_modified (GPANode *node, guint flags)
+gpa_node_emit_modified (GPANode *node)
 {
-	gpa_node_ref (node);
-
-	GPA_NODE_UNSET_FLAGS (node, GPA_NODE_MODIFIED_FLAG);
-
-	g_signal_emit (G_OBJECT (node), node_signals [MODIFIED], 0, flags);
-
-	gpa_node_unref (node);
-}
-
-static gint
-gpa_node_modified_idle_hook (GPANode *node)
-{
-	g_object_set_data (G_OBJECT (node), "idle_id", GUINT_TO_POINTER (0));
-
-	gpa_node_emit_modified (node, 0);
-
-	return FALSE;
-}
-
-void
-gpa_node_request_modified (GPANode *node, guint flags)
-{
-	guint id;
-
-	g_return_if_fail (node != NULL);
-	g_return_if_fail (GPA_IS_NODE (node));
-
-	gpa_enter ("gpa_node_request_modified [%s][%d]", gpa_node_id (node), GPOINTER_TO_INT (node));
-
-	if (GPA_NODE_FLAGS (node) & GPA_NODE_MODIFIED_FLAG) {
-		gpa_leave ("gpa_node_request_modified (Allready marked) [%s]", gpa_node_id (node));
-		return;
-	}
-	
-	GPA_NODE_SET_FLAGS (node, GPA_NODE_MODIFIED_FLAG);
-
-	if (node->parent) {
-		gpa_node_request_modified (node->parent, flags);
-		/* FIXME: Lauris had a return here, so that the parent will only get the request
-		 * and not the child. Figure out if removing this return is ok (Chema)
-		 */
-	}
-	
-	id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (node), "idle_id"));
-
-	if (id == 0) {
-		id = g_idle_add ((GSourceFunc) gpa_node_modified_idle_hook, node);
-		g_object_set_data (G_OBJECT (node), "idle_id", GUINT_TO_POINTER (id));
-	} 
-
-	gpa_leave ("gpa_node_request_modified (3) [%s]", gpa_node_id (node));
-}
-
-/* Public methods */
-
-GPANode *
-gpa_node_ref (GPANode *node)
-{
-	g_return_val_if_fail (node != NULL, NULL);
-	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
-
-	g_object_ref (G_OBJECT (node));
-
-	return node;
-}
-
-GPANode *
-gpa_node_unref (GPANode *node)
-{
-	g_return_val_if_fail (node != NULL, NULL);
-	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
-
-	g_object_unref (G_OBJECT (node));
-
-	return NULL;
+#if 0	
+	g_print ("Emit modified for %s\n", gpa_node_id (node));
+#endif
+	g_signal_emit (G_OBJECT (node), node_signals [MODIFIED], 0, 0);
 }
 
 const guchar *
@@ -323,24 +297,11 @@ gpa_node_id (GPANode *node)
 	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
 
 	if (node->qid)
-		return gpa_quark_to_string (node->qid);
+		return g_quark_to_string (node->qid);
 
 	return NULL;
 }
 
-/* These return referenced node or NULL */
-GPANode *
-gpa_node_get_parent (GPANode *node)
-{
-	g_return_val_if_fail (node != NULL, NULL);
-	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
-
-	if (node->parent)
-		gpa_node_ref (node->parent);
-
-	return node->parent;
-}
-
 /**
  * gpa_node_get_child_from_path:
  * @node: 
@@ -354,16 +315,12 @@ gpa_node_get_parent (GPANode *node)
 GPANode *
 gpa_node_get_child_from_path (GPANode *node, const guchar *path)
 {
-	g_return_val_if_fail (node != NULL, NULL);
-	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
 	g_return_val_if_fail (path != NULL, NULL);
 	g_return_val_if_fail (!*path || isalnum (*path), NULL);
 
 	return gpa_node_lookup (node, path);
 }
 
-/* Basic value manipulation */
-
 guchar *
 gpa_node_get_path_value (GPANode *node, const guchar *path)
 {
@@ -402,6 +359,7 @@ gboolean
 gpa_node_set_path_value (GPANode *parent, const guchar *path, const guchar *value)
 {
 	GPANode *node;
+	gboolean ret;
 
 	g_return_val_if_fail (parent != NULL, FALSE);
 	g_return_val_if_fail (GPA_IS_NODE (parent), FALSE);
@@ -409,44 +367,17 @@ gpa_node_set_path_value (GPANode *parent, const guchar *path, const guchar *valu
 	g_return_val_if_fail (!*path || isalnum (*path), FALSE);
 
 	node = gpa_node_lookup (parent, path);
-
-	if (node) {
-		gboolean ret;
-		ret = gpa_node_set_value (node, value);
-		gpa_node_unref (node);
-		return ret;
+	if (!node) {
+		g_warning ("could not set the value of %s, node not found", path);
+		return FALSE;
 	}
-
-	return FALSE;
+	
+	ret = gpa_node_set_value (node, value);
+	gpa_node_unref (node);
+	return ret;
 }
 
 /* Convenience stuff */
-gboolean
-gpa_node_get_bool_path_value (GPANode *node, const guchar *path, gint *value)
-{
-	guchar *v;
-	
-	g_return_val_if_fail (node != NULL, FALSE);
-	g_return_val_if_fail (GPA_IS_NODE (node), FALSE);
-	g_return_val_if_fail (path != NULL, FALSE);
-	g_return_val_if_fail (!*path || isalnum (*path), FALSE);
-	g_return_val_if_fail (value != NULL, FALSE);
-
-	v = gpa_node_get_path_value (node, path);
-
-	if (v != NULL) {
-		if (!strcasecmp (v, "true") || !strcasecmp (v, "yes") || !strcasecmp (v, "y") || (atoi (v) > 0)) {
-			*value = TRUE;
-			return TRUE;
-		}
-		*value = FALSE;
-		g_free (v);
-		return TRUE;
-	}
-	
-	return FALSE;
-}
-
 gboolean
 gpa_node_get_int_path_value (GPANode *node, const guchar *path, gint *value)
 {
@@ -469,32 +400,6 @@ gpa_node_get_int_path_value (GPANode *node, const guchar *path, gint *value)
 	return FALSE;
 }
 
-gboolean
-gpa_node_get_double_path_value (GPANode *node, const guchar *path, gdouble *value)
-{
-	guchar *v;
-	
-	g_return_val_if_fail (node != NULL, FALSE);
-	g_return_val_if_fail (GPA_IS_NODE (node), FALSE);
-	g_return_val_if_fail (path != NULL, FALSE);
-	g_return_val_if_fail (!*path || isalnum (*path), FALSE);
-	g_return_val_if_fail (value != NULL, FALSE);
-
-	v = gpa_node_get_path_value (node, path);
-
-	if (v != NULL) {
-		gchar *loc;
-		loc = setlocale (LC_NUMERIC, NULL);
-		setlocale (LC_NUMERIC, "C");
-		*value = atof (v);
-		g_free (v);
-		setlocale (LC_NUMERIC, loc);
-		return TRUE;
-	}
-	
-	return FALSE;
-}
-
 #define MM2PT (72.0 / 25.4)
 #define CM2PT (72.0 / 2.54)
 #define IN2PT (72.0)
@@ -516,7 +421,7 @@ gpa_node_get_length_path_value (GPANode *node, const guchar *path, gdouble *valu
 	if (v == NULL)
 		return FALSE;
 	
-	loc = setlocale (LC_NUMERIC, NULL);
+	loc = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
 	*value = strtod (v, &e);
 	if (e) {
@@ -531,61 +436,125 @@ gpa_node_get_length_path_value (GPANode *node, const guchar *path, gdouble *valu
 
 	g_free (v);
 	setlocale (LC_NUMERIC, loc);
+	g_free (loc);
 	
 	return TRUE;
 }
 
-gboolean
-gpa_node_set_bool_path_value (GPANode *node, const guchar *path, gint value)
+
+GPANode *
+gpa_node_ref (GPANode *node)
 {
-	g_return_val_if_fail (node != NULL, FALSE);
-	g_return_val_if_fail (GPA_IS_NODE (node), FALSE);
-	g_return_val_if_fail (path != NULL, FALSE);
-	g_return_val_if_fail (!*path || isalnum (*path), FALSE);
+	g_return_val_if_fail (node != NULL, NULL);
+	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
 
-	return gpa_node_set_path_value (node, path, (value) ? "true" : "false");
-}
+	g_object_ref (G_OBJECT (node));
 
-gboolean
-gpa_node_set_int_path_value (GPANode *node, const guchar *path, gint value)
-{
-	guchar c[64];
-	
-	g_return_val_if_fail (node != NULL, FALSE);
-	g_return_val_if_fail (GPA_IS_NODE (node), FALSE);
-	g_return_val_if_fail (path != NULL, FALSE);
-	g_return_val_if_fail (!*path || isalnum (*path), FALSE);
-
-	g_snprintf (c, 64, "%d", value);
-
-	return gpa_node_set_path_value (node, path, c);
-}
-
-gboolean
-gpa_node_set_double_path_value (GPANode *node, const guchar *path, gdouble value)
-{
-	guchar c[64];
-	gchar *loc;
-	
-	g_return_val_if_fail (node != NULL, FALSE);
-	g_return_val_if_fail (GPA_IS_NODE (node), FALSE);
-	g_return_val_if_fail (path != NULL, FALSE);
-	g_return_val_if_fail (!*path || isalnum (*path), FALSE);
-
-	loc = setlocale (LC_NUMERIC, NULL);
-	setlocale (LC_NUMERIC, "C");
-	g_snprintf (c, 64, "%g", value);
-	setlocale (LC_NUMERIC, loc);
-
-	return gpa_node_set_path_value (node, path, c);
+	return node;
 }
 
 GPANode *
-gpa_defaults (void)
+gpa_node_unref (GPANode *node)
 {
-	static GPANode *config = NULL;
-	
-	config = gpa_config_new ();
+	g_return_val_if_fail (node != NULL, NULL);
+	g_return_val_if_fail (GPA_IS_NODE (node), NULL);
 
-	return config;
+	g_object_unref (G_OBJECT (node));
+
+	return NULL;
+}
+
+GPANode *
+gpa_node_attach (GPANode *parent, GPANode *child)
+{
+	g_return_val_if_fail (parent != NULL, NULL);
+	g_return_val_if_fail (GPA_IS_NODE (parent), NULL);
+	g_return_val_if_fail (child != NULL, NULL);
+	g_return_val_if_fail (GPA_IS_NODE (child), NULL);
+	g_return_val_if_fail (child->parent == NULL, NULL);
+	g_return_val_if_fail (child->next == NULL, NULL);
+
+	child->parent = parent;
+	child->next = parent->children;
+	parent->children = child;
+	
+	return child;
+}
+
+void
+gpa_node_reverse_children (GPANode *node)
+{
+	GPANode *prev = NULL;
+	GPANode *child;
+	
+	g_return_if_fail (node != NULL);
+	g_return_if_fail (GPA_IS_NODE (node));
+
+	child = node->children;
+	while (child) {
+		GPANode *next = child->next;
+		child->next = prev;
+		prev = child;
+		child = next;
+	}
+	node->children = prev;
+}
+
+void
+gpa_node_detach (GPANode *node)
+{
+	GPANode *parent;
+	GPANode *prev;
+
+	g_return_if_fail (node != NULL);
+
+	g_assert (node->parent);
+	g_assert (node->parent->children);
+
+	parent = node->parent;
+	if (parent->children == node) {
+		parent->children = node->next;
+	} else {
+		prev = parent->children;
+		while (prev) {
+			if (prev->next == node)
+				break;
+			prev = prev->next;
+		}
+		g_assert (prev);
+		prev->next = node->next;
+	}
+	
+	node->parent = NULL;
+	node->next = NULL;		
+}
+
+GPANode *
+gpa_node_detach_unref (GPANode *child)
+{
+	g_return_val_if_fail (child != NULL, child);
+	g_return_val_if_fail (GPA_IS_NODE (child), child);
+	
+	gpa_node_detach (child);
+	gpa_node_unref (child);
+
+	return NULL;
+}
+
+void
+gpa_node_detach_unref_children (GPANode *node)
+{
+	GPANode *child;
+
+	g_return_if_fail (node != NULL);
+	g_return_if_fail (GPA_IS_NODE (node));
+
+	child = node->children;
+	while (child) {
+		GPANode *next;
+		next = child->next;
+		gpa_node_detach (child);
+		g_object_unref (G_OBJECT (child));
+		child = next;
+	}
 }
