@@ -27,25 +27,22 @@
 #include <config.h>
 #include <glib.h>
 #include <gmodule.h>
+
 #include <libgnomeprint/gnome-print-module.h>
-#include <libgnomeprint/gpa/gpa-list.h>
-#include <libgnomeprint/gpa/gpa-printer.h>
-#include <libgnomeprint/gpa/gpa-value.h>
-#include <libgnomeprint/gpa/gpa-settings.h>
-#include <libgnomeprint/gpa/gpa-option.h>
-#include <libgnomeprint/gpa/gpa-reference.h>
 #include <libgnomeprint/gpa/gpa-model.h>
+#include <libgnomeprint/gpa/gpa-printer.h>
+#include <libgnomeprint/gpa/gpa-option.h>
+#include <libgnomeprint/gpa/gpa-settings.h>
 
 #include <cups/cups.h>
 #include <cups/ppd.h>
 
-/* Argument order: id, name, vendor */
+/* Argument order: id, name */
 
 xmlChar *model_xml_template = 
 "<?xml version=\"1.0\"?>"
 "<Model Id=\"%s\" Version=\"1.0\">"
 "  <Name>%s</Name>"
-"  <Vendor>%s</Vendor>"
 "  <ModelVersion>0.0.1</ModelVersion>"
 "  <Options>"
 "    <Option Id=\"Transport\">"
@@ -59,7 +56,7 @@ xmlChar *model_xml_template =
 "    <Option Id=\"Output\">"
 "      <Option Id=\"Media\">"
 "        <Option Id=\"PhysicalOrientation\" Type=\"List\" Default=\"R0\">"
-"          <Fill Ref=\"Globals.Media.PhysicalOrientations\"/>"
+"          <Fill Ref=\"Globals.Media.PhysicalOrientation\"/>"
 "        </Option>"
 "        <Key Id=\"Margins\">"
 "          <Key Id=\"Left\" Value=\"0\"/>"
@@ -69,22 +66,13 @@ xmlChar *model_xml_template =
 "        </Key>"
 "      </Option>"
 "    </Option>"
-"    <Option Id=\"Resolution\" Type=\"List\" Default=\"360x360\">"
-"       <Item Id=\"360x360\">"
-"         <Name>360 x 360 DPI</Name>"
-"         <Key Id=\"DPI\" Value=\"360\">"
-"           <Key Id=\"X\" Value=\"360\"/>"
-"           <Key Id=\"Y\" Value=\"360\"/>"
-"         </Key>"
-"       </Item>"
-"    </Option>"
 "    <Option Id=\"Document\">"
 "      <Option Id=\"Page\">"
 "        <Option Id=\"Layout\" Type=\"List\" Default=\"Plain\">"
-"          <Fill Ref=\"Globals.Media.Layouts\"/>"
+"          <Fill Ref=\"Globals.Document.Page.Layout\"/>"
 "        </Option>"
 "        <Option Id=\"LogicalOrientation\" Type=\"List\" Default=\"R0\">"
-"          <Fill Ref=\"Globals.Media.LogicalOrientations\"/>"
+"          <Fill Ref=\"Globals.Document.Page.LogicalOrientation\"/>"
 "        </Option>"
 "        <Option Id=\"Margins\">"
 "          <Option Id=\"Left\" Type=\"String\" Default=\"2 cm\"/>"
@@ -109,18 +97,18 @@ gpa_model_new_from_xml (char *string)
 {
 	GPANode *model;
 	xmlNodePtr root;
-	xmlDocPtr model_xml_doc;
+	xmlDocPtr doc;
 	
-	model_xml_doc = xmlParseDoc (string);
-	if (!model_xml_doc) {
+	doc = xmlParseDoc (string);
+	if (!doc) {
 		g_warning ("Could not parse model xml");
 		return NULL;
 	}
 	
-	root = model_xml_doc->xmlRootNode;
+	root = doc->xmlRootNode;
 	model = gpa_model_new_from_tree (root);
 	
-	xmlFreeDoc (model_xml_doc);
+	xmlFreeDoc (doc);
 	
 	return model;
 }
@@ -128,7 +116,7 @@ gpa_model_new_from_xml (char *string)
 static char *
 get_paper_text (ppd_file_t *ppd, ppd_size_t *size)
 {
-	/* This is dumb and slow and ugly and crappy and I hate myself. */
+	/* This is dumb and slow and ugly and crappy and I hate myself. (Dave)*/
 	int i;
 	
 	for (i = 0; i < ppd->num_groups; i++) {
@@ -151,85 +139,73 @@ get_paper_text (ppd_file_t *ppd, ppd_size_t *size)
 }
 
 static GPANode *
-get_physical_sizes (ppd_file_t *ppd)
+load_paper_sizes (ppd_file_t *ppd, GPANode *parent)
 {
 	ppd_option_t *option;
 	GPANode *node;
 	int i;
 
-	node = gpa_option_list_new ("PhysicalSize");
-	
 	option = ppdFindOption (ppd, "PageSize");
-	if (option && option->defchoice)
-		GPA_OPTION (node)->value = g_strdup (option->defchoice);
+
+	if (!option)
+		/* Should we not add the PageSize if we can't find it
+		 * or create a default one w/ A4 & USLetter? (Chema)
+		 */
+		return NULL;
+	
+	node = gpa_option_list_new (parent, "PhysicalSize",
+				    option->defchoice);
 			
 	for (i = 0; i < ppd->num_sizes; i++) {
-		char *text;
-		char *str;
 		GPANode *size;
-		GPANode *width;
-		GPANode *height;
+		gchar *height;
+		gchar *width;
+		gchar *paper_name;
 
-		text = get_paper_text (ppd, &ppd->sizes[i]);
+		paper_name = get_paper_text (ppd, &ppd->sizes[i]);
+		size = gpa_option_item_new (node, ppd->sizes[i].name, paper_name);
+		g_free (paper_name);
 
-		size = gpa_option_item_new (ppd->sizes[i].name, text);
+		width  = g_strdup_printf ("%d", (int)ppd->sizes[i].width);
+		height = g_strdup_printf ("%d", (int)ppd->sizes[i].length);
 
-		g_free (text);
+		gpa_option_key_new (size, "Width",  width);
+		gpa_option_key_new (size, "Height", height);
 
-		str = g_strdup_printf ("%d", (int)ppd->sizes[i].width);
-		width = gpa_option_string_new ("Width", str);
-		gpa_option_item_append_child (GPA_OPTION_ITEM (size), 
-					      GPA_OPTION (width));
-		gpa_node_unref (width);
-		g_free (str);
-
-		str = g_strdup_printf ("%d", (int)ppd->sizes[i].length);
-		height = gpa_option_string_new ("Height", str);
-		gpa_option_item_append_child (GPA_OPTION_ITEM (size), 
-					      GPA_OPTION (height));
-		gpa_node_unref (height);
-		g_free (str);
-
-		gpa_option_list_append_child (GPA_OPTION_LIST (node), 
-					      GPA_OPTION (size));
-		gpa_node_unref (size);
+		g_free (width);
+		g_free (height);
 	}
-	
+
+	gpa_node_reverse_children (node);
+
 	return node;
 }
 
-static GPANode *
-get_sources (ppd_file_t *ppd)
+static void
+load_paper_sources (ppd_file_t *ppd, GPANode *parent)
 {
 	ppd_option_t *option;
 	GPANode *node;
 	int i;
 
-	node = gpa_option_list_new ("PaperSource");
-	
 	option = ppdFindOption (ppd, "InputSlot");
-	if (option && option->defchoice)
-		GPA_OPTION (node)->value = g_strdup (option->defchoice);
-			
-	for (i = 0; i < option->num_choices; i++) {
-		GPANode *source;
-
-		source = gpa_option_item_new (option->choices[i].choice, option->choices[i].choice);
-		gpa_option_list_append_child (GPA_OPTION_LIST (node), 
-					      GPA_OPTION (source));
-		gpa_node_unref (source);
-	}
+	if (!option)
+		return;
 	
-	return node;
+	node = gpa_option_list_new (parent, "PaperSource",
+				    option->defchoice);
+	
+	for (i = 0; i < option->num_choices; i++)
+		gpa_option_item_new (node,
+				     option->choices[i].choice,
+				     option->choices[i].choice);
 }
 
-static GPANode *
+static GPAModel *
 get_model (const gchar *printer, ppd_file_t *ppd)
 {
 	GPANode *media;
-	GPANode *sizes;
 	GPANode *model;
-	GPANode *sources;
 	GPANode *output;
 	char *xml;
 	char *id;
@@ -237,119 +213,107 @@ get_model (const gchar *printer, ppd_file_t *ppd)
 	id = g_strdup_printf ("Cups-%s-%s", ppd->manufacturer, ppd->nickname);
 
 	model = gpa_model_get_by_id (id, TRUE);
-	if (model)
-		return model;
-
-	xml = g_strdup_printf (model_xml_template, 
-			       id, ppd->nickname, "GNOME"); /*FIXME: this should use a real vendor */
-	model = gpa_model_new_from_xml (xml);
-
-	media = gpa_node_lookup (model, "Options.Output.Media");
-	sizes = get_physical_sizes (ppd);
-	if (sizes) {
-		gpa_option_node_append_child (GPA_OPTION_NODE (media),
-					      GPA_OPTION (sizes));
-		gpa_node_unref (sizes);
+	
+	if (model) {
+		g_free (id);
+		return GPA_MODEL (model);
 	}
-	gpa_node_unref (media);
+
+	xml = g_strdup_printf (model_xml_template, id, ppd->nickname);
+	model = gpa_model_new_from_xml (xml);
+	g_free (xml);
 
 	output = gpa_node_lookup (model, "Options.Output");
-	sources = get_sources (ppd);
-	if (sources) {
-		gpa_option_node_append_child (GPA_OPTION_NODE (output),
-					      GPA_OPTION (sources));
-		gpa_node_unref (sources);
-	}
+	media  = gpa_node_lookup (model, "Options.Output.Media");
+
+	load_paper_sizes   (ppd, media);
+	load_paper_sources (ppd, output);
+
 	gpa_node_unref (output);
+	gpa_node_unref (media);
 
-	
 	g_free (id);
-	g_free (xml);
 	
-	return model;
+	return (GPAModel *)model;
 }
 
 static gboolean
-append_printer (GPAList *printers_list, 
-		const char *printer_name)
+append_printer (GPAList *printers_list, const char *name, gboolean is_default)
 {
-	GPAPrinter *gpa_printer;
-	GPAList *settings_list;
-	GPANode *settings;
-	GPANode *model;
+	GPANode *settings = NULL;
+	GPANode *printer  = NULL;
+	GPAModel *model    = NULL;
 	const char *filename;
 	ppd_file_t *ppd;
+	gboolean retval = FALSE;
 
-	filename = cupsGetPPD (printer_name);
+	filename = cupsGetPPD (name);
 	ppd = ppdOpenFile (filename);
-	if (!ppd) {
-		g_warning ("couldn't get ppd for %s\n", printer_name);
-		return FALSE;
+	if (ppd == NULL)
+		goto append_printer_exit;
+
+	model = get_model (name, ppd);
+	if (model == NULL)
+		goto append_printer_exit;
+
+	settings = gpa_settings_new (model, "Default", "SettIdFromCups");
+	if (settings == NULL)
+		goto append_printer_exit;
+
+	printer = gpa_printer_new (name, name, model, GPA_SETTINGS (settings));
+	if (printer == NULL)
+		goto append_printer_exit;
+
+	if (gpa_node_verify (printer)) {
+		gpa_list_prepend (printers_list, printer);
+		if (is_default)
+			gpa_list_set_default (printers_list, printer);
+		retval = TRUE;
 	}
-	
-	/* Printer */
-	gpa_printer = gpa_printer_new_full (printer_name, printer_name);
-	
-	/* Model */
-	model = get_model (printer_name, ppd);
-	if (!model) {
-		gpa_node_unref (GPA_NODE (gpa_printer));
+
+ append_printer_exit:
+	if (retval == FALSE) {
+		g_warning ("The CUPS printer %s could not be created\n", name);
+
+		my_gpa_node_unref (printer);
+		my_gpa_node_unref (GPA_NODE (model));
+		my_gpa_node_unref (gpa_node_unref (settings));
+	}
+
+	if (ppd) {
 		ppdClose (ppd);
 		unlink (filename);
-		return FALSE;
 	}
-
-	gpa_printer->model = gpa_node_attach (GPA_NODE (gpa_printer), gpa_reference_new (model));
-	gpa_node_unref (GPA_NODE (model));
 	
-	/* Attach a Settings List */
-	settings_list = gpa_list_new (GPA_TYPE_SETTINGS, "Settings", TRUE);
-	gpa_printer->settings = gpa_node_attach (GPA_NODE (gpa_printer), 
-						 GPA_NODE (settings_list));
-
-	/* Attach a Settings and make them the default */
-	settings = gpa_settings_new_from_model_full (GPA_NODE (model), "Default", "SettIdFromCups");
-	gpa_list_prepend     (settings_list, settings);
-	gpa_list_set_default (settings_list, settings);
-
-	/* Verify the printer that we've just created */
-	if (!gpa_node_verify (GPA_NODE (gpa_printer))) {
-		g_warning ("Can't load printer from cups pluging, _verify failed\n");
-		gpa_node_unref (GPA_NODE (gpa_printer));
-		ppdClose (ppd);
-		unlink (filename);		
-		return FALSE;
-	}
-
-	gpa_list_prepend (printers_list, GPA_NODE (gpa_printer));
-
-	ppdClose (ppd);
-	unlink (filename);
-	
-	return TRUE;
+	return retval;
 }
 
 static void
 gnome_print_cups_printer_list_append (gpointer printers_list)
 {
+	gboolean is_default;
+	const gchar *def;
 	gchar **printers;
 	gint num_printers;
 	gint i;
-	
+
 	g_return_if_fail (printers_list != NULL);
 	g_return_if_fail (GPA_IS_LIST (printers_list));
 	
 	num_printers = cupsGetPrinters (&printers);
 	if (num_printers < 1)
 		return;
+
+	def = cupsGetDefault ();
 	
 	for (i = 0; i < num_printers; i ++) {
-		append_printer (GPA_LIST (printers_list), printers [i]);
+		is_default = (def && !strcmp (printers [i], def));
+		append_printer (GPA_LIST (printers_list),
+				printers [i], is_default);
 	}
 	
-	for (i = 0; i < num_printers; i ++) {
+	for (i = 0; i < num_printers; i ++)
 		free (printers[i]);
-	}
 
 	free (printers);
 	
